@@ -29,12 +29,13 @@ pub use manifest_schema::Manifest;
 pub use replication_buffer::manifest_name_for_hostname_path;
 pub use result::Result;
 pub use snapshot::Snapshot;
+pub use snapshot::SnapshotLoadingPolicy;
 
 /// Read the verneuil configuration from this variable by default.
 pub const VERNEUIL_CONFIG_ENV_VAR: &str = "VERNEUIL_CONFIG";
 
 /// Initialization options for the Verneuil VFS.
-#[derive(Clone, Debug, Default, serde::Deserialize)]
+#[derive(Clone, Debug, Default, serde::Deserialize, serde::Serialize)]
 #[serde(default)]
 pub struct Options {
     /// If true, the Verneuil VFS overrides the default sqlite VFS.
@@ -54,8 +55,10 @@ pub struct Options {
     pub replication_spooling_dir_permissions: Option<u32>,
 
     /// List of default replication targets.
-    #[serde(default)]
     pub replication_targets: Vec<replication_target::ReplicationTarget>,
+
+    /// Global policy when loading snapshot data.
+    pub snapshot_loading_policy: SnapshotLoadingPolicy,
 }
 
 #[repr(C)]
@@ -149,6 +152,7 @@ pub fn configure_replication(options: Options) -> Result<()> {
     }
 
     replication_target::set_default_replication_targets(options.replication_targets);
+    snapshot::set_default_snapshot_loading_policy(options.snapshot_loading_policy);
 
     Ok(())
 }
